==> yta-multimedia-0.0.112.tar.gz: yta-multimedia-0.0.112/yta_multimedia/video/edition/effect/moviepy/multiplied_moviepy_effect.py <==
from yta_multimedia.video.edition.effect.moviepy.moviepy_effect import MoviepyEffect
from moviepy.editor import VideoFileClip, CompositeVideoClip, ImageClip, clips_array
from typing import Union
from math import log as math_log, pow as math_pow


class MultipliedMoviepyEffect(MoviepyEffect):
    """
    Generates a clips array with the provided 'clip' being shown
    'times' times (this parameter must be a pow of 4). This
    method has been created to be used internally with our own
    default methods.
    """
    __parameters = {}

    def __init__(self, clip: Union[VideoFileClip, CompositeVideoClip, ImageClip], times: int = None):
        self.__clip = clip

        if times is None:
            times = 4

        self.__parameters['times'] = times

    def process_parameters(self):
        if not self.__parameters['times']:
            self.__parameters['times'] = 4

        if self.__parameters['times'] <= 1 or not math_log(self.__parameters['times'], 4).is_integer():
            # TODO: Raise exception instead of forcing 4 (?)
            self.__parameters['times'] = 4

        return self.__parameters
    
    def apply(self):
        """
        Applies the effect to the provided 'clip' and with the also
        provided parameters needed by this effect.
        """
        if not self.__clip:
            return None
        
        self.process_parameters()
        
        audio = self.__clip.audio
        size = (self.__clip.w, self.__clip.h)

        # We will dynamically build the matrix
        row = []
        group = []
        # 4^3 = 64 => 8x8 = 2^3x2^3  and  4^2 = 16 => 4x4 = 2^2x2^2
        range_limit_value = math_pow(2, math_log(self.__parameters['times'], 4))
        for i in range(int(self.__parameters['times'] / range_limit_value)):
            row = []
            for j in range(int(self.__parameters['times'] / range_limit_value)):
                row.append(self.__clip)
            group.append(row)

        # When building 'clips_array' you sum the resolutions, so if you add four videos
        # of 1920x1080, you'll get one video of 4x(1920x1080) that will be impossible to
        # exported and unexpected. We resize it to avoid this and we don't resize each
        # clip before because they lose quality
        return clips_array(group).resize(size).set_audio(audio)

==> yta-multimedia-0.0.120.tar.gz: yta-multimedia-0.0.120/yta_multimedia/video/edition/effect/moviepy/multiplied_moviepy_effect.py <==
from yta_multimedia.video.edition.effect.video_effect import VideoEffect
from moviepy.editor import VideoFileClip, CompositeVideoClip, ImageClip, clips_array
from typing import Union
from math import log as math_log, pow as math_pow


class MultipliedMoviepyEffect(VideoEffect):
    """
    Generates a clips array with the provided 'clip' being shown
    'times' times (this parameter must be a pow of 4). This
    method has been created to be used internally with our own
    default methods.
    """
    
    @staticmethod
    def apply(clip: Union[VideoFileClip, CompositeVideoClip, ImageClip], times: int = None):
        """
        Applies the effect to the provided 'clip'.
        """
        clip = super().parse_moviepy_video(clip)

        if not times:
            times = 4

        audio = clip.audio
        size = (clip.w, clip.h)

        # We will dynamically build the matrix
        row = []
        group = []
        # 4^3 = 64 => 8x8 = 2^3x2^3  and  4^2 = 16 => 4x4 = 2^2x2^2
        range_limit_value = math_pow(2, math_log(times, 4))
        for i in range(int(times / range_limit_value)):
            row = []
            for j in range(int(times / range_limit_value)):
                row.append(clip)
            group.append(row)

        # When building 'clips_array' you sum the resolutions, so if you add four videos
        # of 1920x1080, you'll get one video of 4x(1920x1080) that will be impossible to
        # exported and unexpected. We resize it to avoid this and we don't resize each
        # clip before because they lose quality
        return clips_array(group).resize(size).set_audio(audio)
